@@ -1,6 +1,6 @@
 # The MIT License
 #
-# Copyright (c) 2015-2019 Sebastian Ramacher
+# Copyright (c) 2015-2021 Sebastian Ramacher
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,20 +20,21 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+from typing import Optional, Type, IO
+from typing_extensions import Literal
+from types import TracebackType
 
+has_fcntl = True
 try:
     import fcntl
     import errno
-
-    has_fcntl = True
 except ImportError:
     has_fcntl = False
 
+has_msvcrt = True
 try:
     import msvcrt
     import os
-
-    has_msvcrt = True
 except ImportError:
     has_msvcrt = False
 
@@ -41,25 +42,30 @@ except ImportError:
 class BaseLock:
     """Base class for file locking"""
 
-    def __init__(self, fileobj, mode=None, filename=None):
-        self.fileobj = fileobj
+    def __init__(self) -> None:
         self.locked = False
 
-    def acquire(self):
+    def acquire(self) -> None:
         pass
 
-    def release(self):
+    def release(self) -> None:
         pass
 
-    def __enter__(self):
+    def __enter__(self) -> "BaseLock":
         self.acquire()
         return self
 
-    def __exit__(self, *args):
+    def __exit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
+    ) -> Literal[False]:
         if self.locked:
             self.release()
+        return False
 
-    def __del__(self):
+    def __del__(self) -> None:
         if self.locked:
             self.release()
 
@@ -67,14 +73,12 @@ class BaseLock:
 class UnixFileLock(BaseLock):
     """Simple file locking for Unix using fcntl"""
 
-    def __init__(self, fileobj, mode=None, filename=None):
-        super().__init__(fileobj)
+    def __init__(self, fileobj, mode: int = 0) -> None:
+        super().__init__()
+        self.fileobj = fileobj
+        self.mode = mode | fcntl.LOCK_EX
 
-        if mode is None:
-            mode = fcntl.LOCK_EX
-        self.mode = mode
-
-    def acquire(self):
+    def acquire(self) -> None:
         try:
             fcntl.flock(self.fileobj, self.mode)
             self.locked = True
@@ -82,7 +86,7 @@ class UnixFileLock(BaseLock):
             if e.errno != errno.ENOLCK:
                 raise e
 
-    def release(self):
+    def release(self) -> None:
         self.locked = False
         fcntl.flock(self.fileobj, fcntl.LOCK_UN)
 
@@ -90,11 +94,12 @@ class UnixFileLock(BaseLock):
 class WindowsFileLock(BaseLock):
     """Simple file locking for Windows using msvcrt"""
 
-    def __init__(self, fileobj, mode=None, filename=None):
-        super().__init__(None)
+    def __init__(self, filename: str) -> None:
+        super().__init__()
         self.filename = f"{filename}.lock"
+        self.fileobj = -1
 
-    def acquire(self):
+    def acquire(self) -> None:
         # create a lock file and lock it
         self.fileobj = os.open(
             self.filename, os.O_RDWR | os.O_CREAT | os.O_TRUNC
@@ -103,13 +108,13 @@ class WindowsFileLock(BaseLock):
 
         self.locked = True
 
-    def release(self):
+    def release(self) -> None:
         self.locked = False
 
         # unlock lock file and remove it
         msvcrt.locking(self.fileobj, msvcrt.LK_UNLCK, 1)
         os.close(self.fileobj)
-        self.fileobj = None
+        self.fileobj = -1
 
         try:
             os.remove(self.filename)
@@ -117,11 +122,14 @@ class WindowsFileLock(BaseLock):
             pass
 
 
-if has_fcntl:
-    FileLock = UnixFileLock
-elif has_msvcrt:
-    FileLock = WindowsFileLock
-else:
-    FileLock = BaseLock
+def FileLock(
+    fileobj: IO, mode: int = 0, filename: Optional[str] = None
+) -> BaseLock:
+    if has_fcntl:
+        return UnixFileLock(fileobj, mode)
+    elif has_msvcrt and filename is not None:
+        return WindowsFileLock(filename)
+    return BaseLock()
+
 
 # vim: sw=4 ts=4 sts=4 ai et
